             .title        "Bandai (INES 70/152) Plugin"

;05/05/07
;BootGod

;07/23/10
;BootGod
;wr_bank value search wasn't scanning large enough range, fixed.

             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
curbank:    .equ 00e2h
banksleft:    .equ 00e3h
tx:           .equ 00e4h

reg:       .equ 00e8h
reg_lo:    .equ 00e8h
reg_hi:    .equ 00e9h


temp_crc:    .equ 00ech

;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Bandai 161/161/32 Carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 64 >> 4,128 >> 4
             .db 64 >> 3,128 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

;register $C000-C0FF layout: MPPPCCCC
;16K PRG at $8000
;8K CHR at $0000
             
;mirroring is either 1-screen (mapper ctrl) or hardwired H/V

             ldx #00h		;x = H mirroring
             
             lda #020h
             jsr wr_ppu		;set PPU to $2000

             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h		;write values 55h and AAh

             lda #024h
             jsr wr_ppu
             lda 2007h		;set PPU to $2400

             lda 2007h
             cmp #055h
             bne got_mir

             lda 2007h
             cmp #0aah
             bne got_mir	;compare first bytes to see if they are the same

             lda #020h
             jsr wr_ppu		;reset PPU to $2000

             lda #0aah
             sta 2007h		;write AAh first this time
             lda #055h
             sta 2007h

             lda #024h
             jsr wr_ppu
             lda 2007h		;set PPU to $2400 again

             lda 2007h
             cmp #0aah
             bne got_mir	;compare again

             inx		;is V mirroring

;check PPU $2800 as well, if that continues to be the same, it is 1-screen

             lda #028h
             jsr wr_ppu
             lda 2007h		;set PPU to $2800

             lda 2007h
             cmp #0aah
             bne got_mir	;compare, if different, is hard-wired

             lda 2007h
             cmp #055h
             bne got_mir

             ldx #4		;is 1-screen mirroring

got_mir:     txa
             jsr send_byte   ;send byte


;get crc of PRG bank 0 (16K)

             lda #0
             jsr wr_bank	;set to PRG bank 0
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             jsr init_crc
             ldy #0
             ldx #64		;init crc for 16K of data (64 pages = 16K)

chk_loop:    lda (address),y	;crc generation loop
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne chk_loop
             inc address_hi
             dex
             bpl chk_loop

             jsr finish_crc	;crc done
             ldx #3

ld_crc:      lda crc0,x		;save for comparision
             sta temp_crc,x
             dex
             bpl ld_crc

;get crc of PRG bank 4

             lda #64
             jsr wr_bank	;set to PRG bank 4 (4 << 4 = 64)
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             jsr init_crc
             ldy #0
             ldx #64		;init crc for 16K of data (64 pages = 16K)

chk_loop2:   lda (address),y	;crc generation loop
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne chk_loop2
             inc address_hi
             dex
             bpl chk_loop2

             jsr finish_crc	;crc done
             ldx #3

;compare crc's of bank 0 and bank 4

chk_crc:     lda crc0,x
             cmp temp_crc,x
             bne crc_fail
             dex
             bpl chk_crc

             ldx #1		;size in pages to send to pc (100h)
             lda #4		
             sta banksleft	;4 16K banks, 64K total
             jmp got_prg

crc_fail:    ldx #2		;size in pages to send to pc (200h)
             lda #8
             sta banksleft	;8 16K banks, 128K total

;send size info

got_prg:     lda #0
             sta curbank
             jsr send_byte	;send num pages lo-byte
             txa
             jsr send_byte	;send num pages hi-byte
             lda #001h
             jsr send_byte	;send type PRG

next_prg:    lda curbank
             asl a
             asl a
             asl a
             asl a
             jsr wr_bank	;write cur bank num (<< 4)
             inc curbank

             lda #0
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             ldy #0
             ldx #64		;setup for 16K bank dump

dump_prg:    lda (address),y
             jsr send_byte
             iny
             bne dump_prg
             inc address_hi
             dex
             bne dump_prg
             dec banksleft
             bne next_prg
             
;determine CHR ROM size (64K or 128K)             

             lda #0
             jsr wr_bank   ;first bank of gfx
             lda #0
             jsr wr_ppu
             jsr init_crc  ;init crc
             lda 2007h

             ldx #020h     ;8K worth of data
             ldy #0

ck_loop:     lda 2007h
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne ck_loop
             dex
             bne ck_loop   ;do all 8K
             
             jsr finish_crc
             ldx #3

ld_crc2:      lda crc0,x
             sta temp_crc,x
             dex
             bpl ld_crc2     ;store CRC
             jsr init_crc   ;crc for next block
             
             
             lda #005h       ;get CRC of 5th bank
             jsr wr_bank
             lda #0
             jsr wr_ppu
             jsr init_crc  ;init crc
             lda 2007h

             ldx #020h
             ldy #0

ck_loop2:    lda 2007h
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne ck_loop2
             dex
             bne ck_loop2   ;do all 8K
             jsr finish_crc
             
             ldy #2
             lda #16
             sta banksleft
             ldx #3

chk_crc2:     lda crc0,x             
             cmp temp_crc,x
             bne crc_dne
             dex
             bpl chk_crc2
             ldy #1
             lda #8
             sta banksleft
             
crc_dne:     lda #0
             jsr send_byte
             tya
             jsr send_byte
             lda #02h
             jsr send_byte   ;send our header

             lda #0
             sta curbank
             sta address_lo
             sta address_hi
             

send_bank:   lda curbank
             jsr wr_bank      ;set bank
             inc curbank
             lda #0
             jsr wr_ppu
             lda 2007h

             ldx #020h
             ldy #0

send_blk:    lda 2007h
             jsr send_byte
             iny
             bne send_blk
             dex
             bne send_blk
             dec banksleft
             bne send_bank

             
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte  ;send end block
             rts

wr_bank:     sta reg		;store value to be written to reg

             lda #0
             sta address_lo
             lda #0C0h
             sta address_hi	;set address to $C000

             ldx #040h		;setup to search last bank (16K)
             ldy #0

wrb_loop:    lda (address),y
             and #07fh		;mask it to just the bits we need to match
             cmp reg		;check if current byte matches what we want to write
             beq found_reg
             iny
             bne wrb_loop
             inc address_hi
             bne wrb_loop

no_reg:      lda reg		;didn't find place to write, try it anyways
             sta 0c000h
             rts

found_reg:   lda (address),y	;found value equal to reg, write it.
             sta (address),y

             rts



             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end